@@ -21,6 +21,7 @@
 STABLE.
 """
 
+import six
 import logging
 from functools import partial
 
@@ -49,6 +50,11 @@ CONN_INTERFACE_ROOM_CONFIG = \
 _logger = logging.getLogger('sugar3.presence.activity')
 
 
+if not hasattr(GObject.ParamFlags, 'READWRITE'):
+    GObject.ParamFlags.READWRITE = GObject.ParamFlags.WRITABLE | \
+        GObject.ParamFlags.READABLE
+
+
 class Activity(GObject.GObject):
     """UI interface for an Activity in the presence service
 
@@ -74,13 +80,13 @@ class Activity(GObject.GObject):
     }
 
     __gproperties__ = {
-        'id': (str, None, None, None, GObject.PARAM_READABLE),
-        'name': (str, None, None, None, GObject.PARAM_READWRITE),
-        'tags': (str, None, None, None, GObject.PARAM_READWRITE),
-        'color': (str, None, None, None, GObject.PARAM_READWRITE),
-        'type': (str, None, None, None, GObject.PARAM_READABLE),
-        'private': (bool, None, None, True, GObject.PARAM_READWRITE),
-        'joined': (bool, None, None, False, GObject.PARAM_READABLE),
+        'id': (str, None, None, None, GObject.ParamFlags.READABLE),
+        'name': (str, None, None, None, GObject.ParamFlags.READWRITE),
+        'tags': (str, None, None, None, GObject.ParamFlags.READWRITE),
+        'color': (str, None, None, None, GObject.ParamFlags.READWRITE),
+        'type': (str, None, None, None, GObject.ParamFlags.READABLE),
+        'private': (bool, None, None, True, GObject.ParamFlags.READWRITE),
+        'joined': (bool, None, None, False, GObject.ParamFlags.READABLE),
     }
 
     def __init__(self, account_path, connection, room_handle=None,
@@ -119,6 +125,11 @@ class Activity(GObject.GObject):
 
     def _start_tracking_properties(self):
         bus = dbus.SessionBus()
+        arg_dict = dict(reply_handler=self.__got_properties_cb,
+                        error_handler=self.__error_handler_cb)
+        if six.PY2:
+            arg_dict = arg_dict.update(utf8_strings=True)
+
         self._get_properties_call = bus.call_async(
             self.telepathy_conn.requested_bus_name,
             self.telepathy_conn.object_path,
@@ -126,9 +137,7 @@ class Activity(GObject.GObject):
             'GetProperties',
             'u',
             (self.room_handle,),
-            reply_handler=self.__got_properties_cb,
-            error_handler=self.__error_handler_cb,
-            utf8_strings=True)
+            arg_dict)
 
         # As only one Activity instance is needed per activity process,
         # we can afford listening to ActivityPropertiesChanged like this.
@@ -139,7 +148,7 @@ class Activity(GObject.GObject):
 
     def __activity_properties_changed_cb(self, room_handle, properties):
         _logger.debug('%r: Activity properties changed to %r' % (self,
-                      properties))
+                                                                 properties))
         self._update_properties(properties)
 
     def __got_properties_cb(self, properties):
@@ -234,7 +243,7 @@ class Activity(GObject.GObject):
         returns list of presence Buddy objects that we can successfully
         create from the buddy object paths that PS has for this activity.
         """
-        return self._joined_buddies.values()
+        return list(self._joined_buddies.values())
 
     def get_buddy_by_handle(self, handle):
         """Retrieve the Buddy object given a telepathy handle.
@@ -295,8 +304,9 @@ class Activity(GObject.GObject):
         channel.connect_to_signal('Closed', self.__text_channel_closed_cb)
 
     def __get_all_members_cb(self, members, local_pending, remote_pending):
-        _logger.debug('__get_all_members_cb %r %r' % (members,
-                      self._text_channel_group_flags))
+        _logger.debug(
+            '__get_all_members_cb %r %r' %
+            (members, self._text_channel_group_flags))
         if self._channel_self_handle in members:
             members.remove(self._channel_self_handle)
 
@@ -554,6 +564,7 @@ class _JoinCommand(_BaseCommand):
         self._finished = False
         self.room_handle = room_handle
         self._global_self_handle = None
+        self._tubes_supported = True
 
     def run(self):
         if self._finished:
@@ -581,7 +592,7 @@ class _JoinCommand(_BaseCommand):
             self.room_handle,
             True,
             reply_handler=self.__create_tubes_channel_cb,
-            error_handler=self.__error_handler_cb,
+            error_handler=self.__tubes_error_handler_cb,
             dbus_interface=CONNECTION)
 
     def __create_text_channel_cb(self, channel_path):
@@ -592,6 +603,15 @@ class _JoinCommand(_BaseCommand):
         Channel(self._connection.requested_bus_name, channel_path,
                 ready_handler=self.__tubes_channel_ready_cb)
 
+    def __tubes_error_handler_cb(self, error):
+        if (error.get_dbus_name() ==
+                'org.freedesktop.Telepathy.Error.NotImplemented'):
+            self._tubes_supported = False
+            self._tubes_ready()
+        else:
+            self._finished = True
+            self.emit('finished', error)
+
     def __error_handler_cb(self, error):
         self._finished = True
         self.emit('finished', error)
@@ -608,7 +628,7 @@ class _JoinCommand(_BaseCommand):
 
     def _tubes_ready(self):
         if self.text_channel is None or \
-                self.tubes_channel is None:
+                (self._tubes_supported and self.tubes_channel is None):
             return
 
         _logger.debug('%r: finished setting up tubes' % self)
@@ -616,8 +636,9 @@ class _JoinCommand(_BaseCommand):
         self._add_self_to_channel()
 
     def __text_channel_group_flags_changed_cb(self, added, removed):
-        _logger.debug('__text_channel_group_flags_changed_cb %r %r' % (added,
-                      removed))
+        _logger.debug(
+            '__text_channel_group_flags_changed_cb %r %r' %
+            (added, removed))
         self.text_channel_group_flags |= added
         self.text_channel_group_flags &= ~removed
 
